# copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
# this program is free software: please refer to the BSD type license enclosed in this package

set rcsId {$Id: blt2d.tcl,v 1.6 1998/03/08 19:01:19 jfontain Exp $}

class blt2DViewer {

    # use pie colors for consistency
    set blt2DViewer::(colors) {#7FFFFF #7FFF7F #FF7F7F #FFFF7F #7F7FFF #FFBF00 #BFBFBF #FF7FFF #FFFFFF}

    proc blt2DViewer {this path} viewer {} {
        $path configure -plotbackground $widget::(default,ButtonBackgroundColor)
        $path yaxis configure -title {} -min 0 -tickfont $font::(smallNormal)
        $path legend configure -borderwidth 1 -font $font::(mediumNormal)

        # allow dropping of data cells
        set blt2DViewer::($this,drop) [new dropSite\
            -path $path -formats DATACELL -command "viewer::view $this \$dragSite::data(DATACELL)"\
        ]

        set blt2DViewer::($this,drag) [new dragSite -path $path -validcommand "blt2DViewer::validateDrag $this"]
        dragSite::provide $blt2DViewer::($this,drag) OBJECTS "blt2DViewer::dragData $this"                ;# provide list of objects

        set blt2DViewer::($this,elements) {}                                             ;# initialize list of data line identifiers
        set blt2DViewer::($this,selectedElements) {}
        bind $path <ButtonRelease-1> "blt2DViewer::select $this 0 %x %y"
        bind $path <Control-ButtonRelease-1> "blt2DViewer::select $this 1 %x %y"

        set blt2DViewer::($this,colorIndex) 0
        set blt2DViewer::($this,path) $path
    }

    proc ~blt2DViewer {this} {
        catch {trace vdelete $blt2DViewer::($this,array)(updates) w "blt2DViewer::update $this"}          ;# eventually remove trace
        delete $blt2DViewer::($this,drag) $blt2DViewer::($this,drop)
        eval delete $blt2DViewer::($this,elements)                                                       ;# delete existing elements
    }

    proc supportedTypes {this} {
        return {integer real}
    }

    proc dragData {this format} {                                                   ;# only OBJECTS format is supported at this time
        if {[llength $blt2DViewer::($this,selectedElements)]>0} {
            return $blt2DViewer::($this,selectedElements)                               ;# return selected elements if there are any
        } elseif {[llength $blt2DViewer::($this,elements)]==0} {
            return $this                                                           ;# return graph itself if it contains no elements
        } else {
            return {}                                                                                    ;# return nothing otherwise
        }
    }

    proc validateDrag {this x y} {
        if {[llength $blt2DViewer::($this,elements)]>0} {                                         ;# if there are displayed elements
            return [string length [$blt2DViewer::($this,path) legend get @$x,$y]]                             ;# good if in a legend
        } else {
            return 1                                                                        ;# any point in an empty viewer is valid
        }
    }

    proc select {this control x y} {
        set element [$blt2DViewer::($this,path) legend get @$x,$y]
        if {[string length $element]==0} return                                                                   ;# not in a legend
        if {$control} {                                                                                  ;# toggle element selection
            set index [lsearch -exact $blt2DViewer::($this,selectedElements) $element]
            if {$index<0} {
                lappend blt2DViewer::($this,selectedElements) $element
                $blt2DViewer::($this,path) legend activate $element
            } else {
                set blt2DViewer::($this,selectedElements) [lreplace $blt2DViewer::($this,selectedElements) $index $index]
                $blt2DViewer::($this,path) legend deactivate $element
            }
        } else {                                                                                              ;# select sole element
            set blt2DViewer::($this,selectedElements) [list $element]
            $blt2DViewer::($this,path) legend deactivate *
            $blt2DViewer::($this,path) legend activate $element
        }
    }

    proc monitorCell {this array row column} {
        if {![info exists blt2DViewer::($this,array)]} {                                    ;# the module array should stay the same
            trace variable ${array}(updates) w "blt2DViewer::update $this"                    ;# refresh as module data gets updated
            set blt2DViewer::($this,array) $array
        }
        set cell ${array}($row,$column)
        foreach element $blt2DViewer::($this,elements) {
            if {[string compare $blt2DViewer::($this,data,$element) $cell]==0} return                      ;# already graphed, abort
        }

### assume that first column in row uniquely identifies row ###
        set element [newElement $this $blt2DViewer::($this,path) -label "[set ${array}($row,0)] [set ${array}($column,label)]"]
        switched::configure $element -color [lindex $blt2DViewer::(colors) $blt2DViewer::($this,colorIndex)]
        switched::configure $element -deletecommand "blt2DViewer::deletedElement $this $element"  ;# keep track of element existence
        # circle through colors
        set blt2DViewer::($this,colorIndex) [expr {($blt2DViewer::($this,colorIndex)+1)%[llength $blt2DViewer::(colors)]}]
        set blt2DViewer::($this,data,$element) $cell
        lappend blt2DViewer::($this,elements) $element                                                     ;# register new data line
    }

    proc deletedElement {this element} {
        ldelete blt2DViewer::($this,elements) $element
        catch {ldelete blt2DViewer::($this,selectedElements) $element}
        if {[llength $blt2DViewer::($this,elements)]==0} {
            delete $this                                                            ;# self destruct when there are no more elements
        }
    }

    virtual proc update {this args}            ;# update display in derived class, using cells data. ignore eventual trace arguments

    virtual proc newElement {this path args}                                       ;# let derived class create an element of its own
}
